unit fPinecone;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics,
  Controls, Forms, Dialogs, ExtCtrls, ComCtrls, StdCtrls,
  // sgc
  sgcHTTP_API_Pinecone;

type
  TfrmOpenAI = class(TForm)
    PageControl1: TPageControl;
    tabIndexes: TTabSheet;
    pnlTop: TPanel;
    txtApiKey: TEdit;
    lblApiKey: TLabel;
    memoLog: TMemo;
    btnIndexesList: TButton;
    btnCreateIndex: TButton;
    txtCreateIndexName: TEdit;
    tabVectors: TTabSheet;
    btnVectorsDescribeIndexStats: TButton;
    Label1: TLabel;
    Label2: TLabel;
    txtCreateIndexDimensions: TEdit;
    btnIndexDescribe: TButton;
    Label3: TLabel;
    txtIndexDescribeName: TEdit;
    btnIndexDelete: TButton;
    Label4: TLabel;
    txtIndexDeleteName: TEdit;
    btnIndexConfigure: TButton;
    Label5: TLabel;
    Label6: TLabel;
    txtIndexConfigureReplicas: TEdit;
    txtIndexConfigureName: TEdit;
    Label7: TLabel;
    txtIndexConfigurePodtype: TEdit;
    txtVectorsIndexName: TEdit;
    Label8: TLabel;
    Label9: TLabel;
    txtVectorsProjectId: TEdit;
    btnVectorsQuery: TButton;
    txtVectorsQueryVector: TEdit;
    Label10: TLabel;
    btnVectorsDelete: TButton;
    txtVectorsDeleteVectorId: TEdit;
    Label11: TLabel;
    btnVectorsUpsert: TButton;
    Label12: TLabel;
    txtVectorsUpsertVector: TEdit;
    Label13: TLabel;
    txtVectorsUpsertVectorId: TEdit;
    Label14: TLabel;
    txtVectorsFetchVectorId: TEdit;
    btnVectorsFetch: TButton;
    btnVectorsUpdate: TButton;
    Label15: TLabel;
    txtVectorsUpdateVectorId: TEdit;
    Label16: TLabel;
    txtVectorsUpdateVector: TEdit;
    tabCollections: TTabSheet;
    btnListCollections: TButton;
    btnCollectionsCreate: TButton;
    Label17: TLabel;
    txtCollectionsCreateName: TEdit;
    Label18: TLabel;
    txtCollectionsCreateSource: TEdit;
    txtCollectionsDescribeName: TEdit;
    Label19: TLabel;
    btnCollectionsDescribe: TButton;
    btnCollectionsDelete: TButton;
    Label20: TLabel;
    txtCollectionsDeleteName: TEdit;
    Label21: TLabel;
    txtEnvironment: TEdit;
    procedure btnCollectionsCreateClick(Sender: TObject);
    procedure btnCollectionsDeleteClick(Sender: TObject);
    procedure btnCollectionsDescribeClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnIndexesListClick(Sender: TObject);
    procedure btnCreateIndexClick(Sender: TObject);
    procedure btnIndexConfigureClick(Sender: TObject);
    procedure btnIndexDeleteClick(Sender: TObject);
    procedure btnIndexDescribeClick(Sender: TObject);
    procedure btnListCollectionsClick(Sender: TObject);
    procedure btnVectorsDeleteClick(Sender: TObject);
    procedure btnVectorsDescribeIndexStatsClick(Sender: TObject);
    procedure btnVectorsFetchClick(Sender: TObject);
    procedure btnVectorsQueryClick(Sender: TObject);
    procedure btnVectorsUpdateClick(Sender: TObject);
    procedure btnVectorsUpsertClick(Sender: TObject);
    procedure txtApiKeyChange(Sender: TObject);
    procedure txtEnvironmentChange(Sender: TObject);
  private
    procedure DoLoadSettings;
    procedure DoSaveSettings;
  private
    fPinecone: TsgcHTTP_API_Pinecone;
    function GetPinecone: TsgcHTTP_API_Pinecone;
  public
    property Pinecone: TsgcHTTP_API_Pinecone read GetPinecone write fPinecone;
  public
    procedure DoLog(const aText: string);
  end;

var
  frmOpenAI: TfrmOpenAI;

implementation

{$R *.dfm}

uses
  INIFiles,
  // sgc
  sgcAI_Helpers;

procedure TfrmOpenAI.btnCollectionsCreateClick(Sender: TObject);
begin
  DoLog(Pinecone.CollectionCreate(txtCollectionsCreateName.Text, txtCollectionsCreateSource.Text));
end;

procedure TfrmOpenAI.btnCollectionsDeleteClick(Sender: TObject);
begin
  DoLog(Pinecone.CollectionDelete(txtCollectionsDeleteName.Text));
end;

procedure TfrmOpenAI.btnCollectionsDescribeClick(Sender: TObject);
begin
  DoLog(Pinecone.CollectionDescribe(txtCollectionsDescribeName.Text));
end;

procedure TfrmOpenAI.FormDestroy(Sender: TObject);
begin
  DoSaveSettings;
end;

procedure TfrmOpenAI.FormCreate(Sender: TObject);
begin
  DoLoadSettings;
end;

procedure TfrmOpenAI.btnIndexesListClick(Sender: TObject);
begin
  DoLog(Pinecone.IndexesList);
end;

procedure TfrmOpenAI.btnCreateIndexClick(Sender: TObject);
var
  oParameters: TsgcHTTPPineconeIndexCreate;
begin
  oParameters := TsgcHTTPPineconeIndexCreate.Create;
  Try
    oParameters.Name := txtCreateIndexName.Text;
    oParameters.Dimension := StrToInt(txtCreateIndexDimensions.Text);

    DoLog(Pinecone.IndexCreate(oParameters));
  Finally
    FreeAndNil(oParameters);
  End;
end;

procedure TfrmOpenAI.btnIndexConfigureClick(Sender: TObject);
begin
  DoLog(Pinecone.IndexConfigure(txtIndexConfigureName.Text,
    StrToInt(txtIndexConfigureReplicas.Text), txtIndexConfigurePodtype.Text));
end;

procedure TfrmOpenAI.btnIndexDeleteClick(Sender: TObject);
begin
  DoLog(Pinecone.IndexDelete(txtIndexDeleteName.Text));
end;

procedure TfrmOpenAI.btnIndexDescribeClick(Sender: TObject);
begin
  DoLog(Pinecone.IndexDescribe(txtIndexDescribeName.Text));
end;

procedure TfrmOpenAI.btnListCollectionsClick(Sender: TObject);
begin
  DoLog(Pinecone.CollectionsList);
end;

procedure TfrmOpenAI.btnVectorsDeleteClick(Sender: TObject);
var
  oParams: TsgcHTTPPineconeVectorDelete;
begin
  oParams := TsgcHTTPPineconeVectorDelete.Create;
  Try
    oParams.Ids.Add(txtVectorsDeleteVectorId.Text);
    DoLog(Pinecone.VectorsDelete(txtVectorsIndexName.Text, txtVectorsProjectId.Text, oParams));
  Finally
    oParams.Free;
  End;
end;

procedure TfrmOpenAI.btnVectorsDescribeIndexStatsClick(Sender: TObject);
begin
  DoLog(PineCone.VectorsDescribeIndexStats(txtVectorsIndexName.text, txtVectorsProjectId.text));
end;

procedure TfrmOpenAI.btnVectorsFetchClick(Sender: TObject);
var
  oIds: TStringList;
begin
  oIds := TStringList.Create;
  Try
    oIds.Add(txtVectorsFetchVectorId.Text);
    DoLog(Pinecone.VectorsFetch(txtVectorsIndexName.Text, txtVectorsProjectId.Text, oIds));
  Finally
    oIds.Free;
  End;
end;

procedure TfrmOpenAI.btnVectorsQueryClick(Sender: TObject);
var
  oParams: TsgcHTTPPineconeVectorQuery;
begin
  oParams := TsgcHTTPPineconeVectorQuery.Create;
  Try
    oParams.Vector := sgcStringToArrayOfDouble(txtVectorsQueryVector.Text);
    DoLog(Pinecone.VectorsQuery(txtVectorsIndexName.Text, txtVectorsProjectId.Text, oParams));
  Finally
    oParams.Free;
  End;
end;

procedure TfrmOpenAI.btnVectorsUpdateClick(Sender: TObject);
var
  oParams: TsgcHTTPPineconeVectorUpdate;
begin
  oParams := TsgcHTTPPineconeVectorUpdate.Create;
  Try
    oParams.Id := txtVectorsUpdateVectorId.text;
    oParams.Values := sgcStringToArrayOfDouble(txtVectorsUpdateVector.Text);
    DoLog(Pinecone.VectorsUpdate(txtVectorsIndexName.Text, txtVectorsProjectId.Text, oParams));
  Finally
    oParams.Free;
  End;
end;

procedure TfrmOpenAI.btnVectorsUpsertClick(Sender: TObject);
var
  oParams: TsgcHTTPPineconeVectorUpserts;
  oVectors: TsgcArrayOfVectorUpsert;
begin
  oParams := TsgcHTTPPineconeVectorUpserts.Create;
  Try
    SetLength(oVectors, 1);
    oVectors[0] := TsgcHTTPPineconeVectorUpsert.Create;
    oVectors[0].Id := txtVectorsUpsertVectorId.text;
    oVectors[0].Values := sgcStringToArrayOfDouble(txtVectorsUpsertVector.Text);
    oParams.Vectors := oVectors;
    DoLog(Pinecone.VectorsUpsert(txtVectorsIndexName.Text, txtVectorsProjectId.Text, oParams));
  Finally
    oParams.Free;
  End;
end;

procedure TfrmOpenAI.DoLoadSettings;
var
  oINI: TINIFile;
begin
  oINI := TINIFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  Try
    txtApiKey.Text := oINI.ReadString('PINECONE', 'API_KEY', '');
    txtEnvironment.Text := oINI.ReadString('PINECONE', 'ENVIRONMENT', 'us-west4-gcp-free');
    txtVectorsIndexName.Text := oINI.ReadString('PINECONE.INDEX', 'INDEX_NAME', '');
    txtVectorsProjectId.Text := oINI.ReadString('PINECONE.INDEX', 'PROJECT_ID', '');
  Finally
    oINI.Free;
  End;
end;

procedure TfrmOpenAI.DoLog(const aText: string);
begin
  memoLog.Lines.Text := aText;
end;

procedure TfrmOpenAI.DoSaveSettings;
var
  oINI: TINIFile;
begin
  oINI := TINIFile.Create(ChangeFileExt(Application.ExeName, '.ini'));
  Try
    oINI.Writestring('PINECONE', 'API_KEY', txtApiKey.Text);
    oINI.Writestring('PINECONE', 'ENVIRONMENT', txtEnvironment.Text);
    oINI.WriteString('PINECONE.INDEX', 'INDEX_NAME', txtVectorsIndexName.Text);
    oINI.WriteString('PINECONE.INDEX', 'PROJECT_ID', txtVectorsProjectId.Text);
  Finally
    oINI.Free;
  End;
end;

function TfrmOpenAI.GetPinecone: TsgcHTTP_API_Pinecone;
begin
  if not Assigned(fPinecone) then
    fPinecone := TsgcHTTP_API_Pinecone.Create(self);
  Result := fPinecone;
end;

procedure TfrmOpenAI.txtApiKeyChange(Sender: TObject);
begin
  Pinecone.PineconeOptions.ApiKey := txtApiKey.Text;
end;

procedure TfrmOpenAI.txtEnvironmentChange(Sender: TObject);
begin
  Pinecone.PineconeOptions.Environment := txtEnvironment.Text;
end;

end.
